@@ -1,19 +1,26 @@
-import click
-import sys
+"""TODO: write docstring"""
+from __future__ import annotations
 from pathlib import Path
+import sys
+from typing import TYPE_CHECKING
+
+import click
 
 from . import __version__, __repo__
-from .template import ProjectTemplate, PackageLinker
-from .filesystem import (ProjectPath, CONFIG_PATH, JINJA_PATH,
+from .error import BasePathError, SubcommandError, LaTeXCompileError
+from .export import create_archive
+from .filesystem import (ProjectInfo, JINJA_PATH,
         SHUTIL_ARCHIVE_FORMATS, SHUTIL_ARCHIVE_SUFFIX_MAP,
         format_linker, macro_linker, citation_linker, template_linker)
-from .export import create_archive
-from .error import BasePathError, SubcommandError, LaTeXCompileError
-from .term import err_echo
 from .process import subproc_run, compile_tex, get_github_api_token
+from .template import LoadTemplate, InitTemplate, PackageLinker
+from .term import err_echo
 
+if TYPE_CHECKING:
+    from typing import Optional, Iterable
 
 class CatchInternalExceptions(click.Group):
+    """TODO: write"""
     def __call__(self, *args, **kwargs):
         try:
             return self.main(*args, **kwargs)
@@ -22,13 +29,6 @@ class CatchInternalExceptions(click.Group):
             err_echo(err)
             sys.exit(1)
 
-class ProjectInfo(ProjectPath):
-    def __init__(self, proj_dir, dry_run, verbose):
-        self.dry_run = dry_run
-        self.force = False
-        self.verbose = verbose or dry_run # always verbose during dry_run
-        super().__init__(proj_dir)
-
 @click.group(cls=CatchInternalExceptions)
 @click.version_option(prog_name="tpr (texproject)")
 @click.option(
@@ -37,7 +37,8 @@ class ProjectInfo(ProjectPath):
         show_default=True,
         help='working directory',
         type=click.Path(
-            exists=True, file_okay=False, dir_okay=True, writable=True, path_type=Path))
+            exists=True, file_okay=False, dir_okay=True, writable=True,
+            path_type=Path))
 @click.option(
         '-n', '--dry-run',
         'dry_run',
@@ -48,17 +49,18 @@ class ProjectInfo(ProjectPath):
         default=True,
         help='Be verbose')
 @click.pass_context
-def cli(ctx, proj_dir, dry_run, verbose):
+def cli(ctx, proj_dir: Path, dry_run: bool, verbose: bool) -> None:
+    """TODO: write"""
     ctx.obj = ProjectInfo(proj_dir, dry_run, verbose)
 
 
 @cli.command()
 @click.argument('template')
-@click.option('--citation','-c',
+@click.option('--citation','-c', 'citations',
         multiple=True,
         help="include citation file")
 @click.pass_obj
-def init(proj_info, template, citation):
+def init(proj_info: ProjectInfo, template: str, citations: Iterable[str]) -> None:
     """Initialize a new project in the current directory. The project is
     created using the template with name TEMPLATE and placed in the output
     folder OUTPUT.
@@ -67,69 +69,71 @@ def init(proj_info, template, citation):
     intermediate directories are automatically constructed."""
     proj_info.validate(exists=False)
 
-    proj_gen = ProjectTemplate.load_from_template(
+    proj_gen = InitTemplate(
             template,
-            citation)
+            citations)
 
     proj_gen.create_output_folder(proj_info)
     proj_gen.write_tpr_files(proj_info)
 
 
 @cli.command()
-@click.option('--local', 'config_file', flag_value='local', default=True,
+@click.option('--template', 'config_file', flag_value='template', default=True,
         help="Edit local project configuration.")
-@click.option('--user', 'config_file', flag_value='user',
-        help="Edit user information.")
-@click.option('--system', 'config_file', flag_value='system',
-        help="Edit global system configuration.")
+@click.option('--local', 'config_file', flag_value='local',
+        help="Edit local project configuration.")
+@click.option('--global', 'config_file', flag_value='global',
+        help="Edit global configuration.")
 @click.pass_obj
-def config(proj_info, config_file):
+def config(proj_info: ProjectInfo, config_file: str) -> None:
     """Edit texproject configuration files. This opens the corresponding file
     in your $EDITOR. By default, edit the project configuration file; this
-    requires the current directory (or the directory specified by -C) to be
-    a texproject directory.
+    requires the current directory (or the directory specified by -C) to be a
+    texproject directory.
 
-    Note that this command does not replace existing macro files. See the
-    `tpr import` command for this functionality.
+    Note that this command does not replace existing macro files. See the `tpr
+    import` command for this functionality.
     """
     match config_file:
-        case 'local':
+        case 'template':
             proj_info.validate(exists=True)
 
-            click.edit(filename=str(proj_info.config))
+            click.edit(filename=str(proj_info.template))
 
-            proj_info = ProjectTemplate.load_from_project(proj_info)
-            proj_info.write_tpr_files(proj_info)
+            proj_gen = LoadTemplate(proj_info)
+            proj_gen.write_tpr_files(proj_info)
 
-        case 'user':
-            click.edit(filename=str(CONFIG_PATH.user))
+        case 'local':
+            click.edit(filename=str(proj_info.config.local_path))
 
-        case 'system':
-            click.edit(filename=str(CONFIG_PATH.system))
+        case 'global':
+            click.edit(filename=str(proj_info.config.global_path))
 
 
 @cli.command('import')
-@click.option('--macro', multiple=True,
+@click.option('--macro', 'macros', multiple=True,
         help="macro file")
-@click.option('--citation', multiple=True,
+@click.option('--citation', 'citations', multiple=True,
         help="citation file")
-@click.option('--format', default=None,
+@click.option('--format', 'format_', default=None,
         help="format file")
 @click.option('--path/--no-path', default=False,
         help="files given as absolute paths")
 @click.pass_obj
-def import_(proj_info, macro, citation, format, path):
-    """Import macro, citation, and format files. This command will replace existing
-    files. When called with the --path option, the arguments are interpreted as paths.
-    This is convenient when importing packages which are not installed in the texproject
-    directories.
+def import_(proj_info: ProjectInfo,
+        macros: Iterable[str], citations: Iterable[str], format_: Optional[str],
+        path: bool) -> None:
+    """Import macro, citation, and format files. This command will replace
+    existing files. When called with the --path option, the arguments are
+    interpreted as paths. This is convenient when importing packages which are
+    not installed in the texproject directories.
     """
     proj_info.validate(exists=True)
 
     linker = PackageLinker(proj_info, force=True, is_path=path)
-    linker.link_macros(macro)
-    linker.link_citations(citation)
-    linker.link_format(format)
+    linker.link_macros(macros)
+    linker.link_citations(citations)
+    linker.link_format(format_)
 
 
 
@@ -143,28 +147,33 @@ def import_(proj_info, macro, citation, format, path):
         type=click.Path(
             exists=False, writable=True, path_type=Path))
 @click.pass_obj
-def validate(proj_info, output, logfile):
-    """Check project for compilation errors. You can save the resulting pdf by specifying
-    the '--output' argument, and similarly save the log file with the '--logfile' argument.
-    Note that these options, if specified, will automatically overwrite existing files.
+def validate(proj_info: ProjectInfo, output: Path, logfile: Path) -> None:
+    """Check project for compilation errors. You can save the resulting pdf by
+    specifying the '--output' argument, and similarly save the log file with
+    the '--logfile' argument. Note that these options, if specified, will
+    automatically overwrite existing files.
 
-    Compilation requires the 'latexmk' program to be installed and accessible to this program.
-    Compilation is done by executing the command
+    Compilation requires the 'latexmk' program to be installed and accessible
+    to this program. Compilation is done by executing the command
 
     $ latexmk -pdf -interaction=nonstopmode
 
-    You can specify additional options in 'config.system.latex_compile_options'.
+    You can specify additional options in
+    'config.system.latex_compile_options'.
     """
+    proj_info.validate(exists=True)
     with proj_info.temp_subpath() as build_dir:
         build_dir.mkdir()
-        compile_tex(proj_info, outdir=build_dir, output_map={'.pdf': output, '.log': logfile})
+        compile_tex(proj_info,
+                outdir=build_dir,
+                output_map={'.pdf': output, '.log': logfile})
 
 
-def validate_exists(ctx, _, path):
+def validate_exists(ctx, _, path) -> Path:
+    """TODO: write"""
     if not ctx.obj.force and path.exists():
         raise click.BadParameter('file exists. Use -f / --force to overwrite.')
-    else:
-        return path
+    return path
 @cli.command()
 @click.option('--force/--no-force','-f/-F', default=False,
         help="overwrite files")
@@ -183,15 +192,17 @@ def validate_exists(ctx, _, path):
             exists=False, writable=True, path_type=Path),
         callback=validate_exists)
 @click.pass_obj
-def archive(proj_info, force, compression, inc, output):
-    """Create a compressed export with name OUTPUT. If the 'arxiv' or 'build' options are
-    chosen, 'latexmk' is used to compile additional required files. Run 'tpr validate --help'
-    for more information.
+def archive(proj_info: ProjectInfo, force: bool, compression: str, inc: str, output: Path) -> None:
+    """Create a compressed export with name OUTPUT. If the 'arxiv' or 'build'
+    options are chosen, 'latexmk' is used to compile additional required files.
+    Run 'tpr validate --help' for more information.
 
-    The --format option specifies the format of the resulting archive. If unspecified, the format is
-    inferred from the resulting filename if possible. Otherwise, the output format is 'tar'.
+    The --format option specifies the format of the resulting archive. If
+    unspecified, the format is inferred from the resulting filename if
+    possible. Otherwise, the output format is 'tar'.
 
-    Note: if the format is not inferred from the filename, the archive file suffix is appended automatically.
+    Note: if the format is not inferred from the filename, the archive file
+    suffix is appended automatically.
 
     \b
     File inclusion modes:
@@ -225,7 +236,7 @@ def archive(proj_info, force, compression, inc, output):
 
 @cli.group()
 @click.pass_obj
-def git(proj_info):
+def git(proj_info: ProjectInfo) -> None:
     """Subcommand to manage git files.
     """
     proj_info.validate(exists=True)
@@ -254,27 +265,30 @@ def git(proj_info):
         help='Create issues page',
         default=False)
 @click.pass_obj
-def git_init(proj_info, repo_name, repo_desc, vis, wiki, issues):
-    """Initialize git and a corresponding GitHub repository. If called with no options,
-    this command will interactively prompt you in order to initialize the repo correctly.
-    This command also creates a GitHub action with automatically compiles and releases the
-    main .pdf file for tagged releases.
+def git_init(proj_info: ProjectInfo, repo_name: str, repo_desc: str,
+        vis: str,
+        wiki: bool, issues: bool) -> None:
+    """Initialize git and a corresponding GitHub repository. If called with no
+    options, this command will interactively prompt you in order to initialize
+    the repo correctly. This command also creates a GitHub action with
+    automatically compiles and releases the main .pdf file for tagged releases.
 
     If you have specified 'config.user.github.archive', the
-    GitHub action will also automatically push the build files to the corresponding folder
-    in the specified repository. In order for this to work, you must provide an access token must
-    with at least repo privileges. This can be done (in order of priority) by
+    GitHub action will also automatically push the build files to the
+    corresponding folder in the specified repository. In order for this to
+    work, you must provide an access token must with at least repo privileges.
+    This can be done (in order of priority) by
 
     1) setting the environment variable $API_TOKEN_GITHUB, or
 
     2) setting the 'github.keyring' settings in the system configuration.
 
-    Otherwise, the token will default to the empty string. The access token is not required for
-    the build action functionality.
+    Otherwise, the token will default to the empty string. The access token is
+    not required for the build action functionality.
     """
     proj_info.validate_git(exists=False)
 
-    proj_gen = ProjectTemplate.load_from_project(proj_info)
+    proj_gen = LoadTemplate(proj_info)
     proj_gen.write_git_files(proj_info)
 
     # initialize repo
@@ -307,16 +321,19 @@ def git_init(proj_info, repo_name, repo_desc, vis, wiki, issues):
 
     subproc_run(proj_info,
             ['gh', 'secret', 'set', 'API_TOKEN_GITHUB',
-                '-b', get_github_api_token(),
+                '-b', get_github_api_token(proj_info),
                 '-r', repo_name])
 
 @git.command()
+@click.option('--force/--no-force','-f/-F', default=False,
+        help="overwrite files")
 @click.pass_obj
-def init_files(proj_info):
+def init_files(proj_info: ProjectInfo, force: bool) -> None:
     """Initialize git files, but do not create the repository."""
-    proj_info.validate_git(exists=False)
-    proj_gen = ProjectTemplate.load_from_project(proj_info)
-    proj_gen.write_git_files(proj_info)
+    if not force:
+        proj_info.validate_git(exists=False)
+    proj_gen = LoadTemplate(proj_info)
+    proj_gen.write_git_files(proj_info, force=force)
 
 
 @git.command()
@@ -325,23 +342,23 @@ def init_files(proj_info):
         help='Name of the repository',
         type=str)
 @click.pass_obj
-def set_archive(proj_info, repo_name):
+def set_archive(proj_info: ProjectInfo, repo_name: str) -> None:
     """Set the GitHub secret and archive repository.
     """
-    proj_gen = ProjectTemplate.load_from_project(proj_info)
+    proj_gen = LoadTemplate(proj_info)
     proj_gen.write_template_with_info(proj_info,
             JINJA_PATH.build_latex,
             proj_info.build_latex,
             force=True)
     subproc_run(proj_info,
             ['gh', 'secret', 'set', 'API_TOKEN_GITHUB',
-                '-b', get_github_api_token(),
+                '-b', get_github_api_token(proj_info),
                 '-r', repo_name])
 
-@cli.command()
+@cli.command('list')
 @click.argument('res_class',
         type=click.Choice(['citation', 'macro', 'format', 'template']))
-def list(res_class):
+def list_(res_class: str) -> None:
     """Retrieve program and template information."""
 
     linker_map = {
@@ -356,36 +373,41 @@ def list(res_class):
 
 @cli.group()
 @click.pass_obj
-def upgrade(proj_info):
+def upgrade(proj_info: ProjectInfo) -> None:
     """Various utilities to facilitate the upgrading of old repositories.
     Warning: these commands are destructive!
     """
     proj_info.validate(exists=True)
 
 
-@upgrade.command()
+@upgrade.command('config')
 @click.pass_obj
-def yaml(proj_info):
+def config_(proj_info: ProjectInfo) -> None:
     """Update configuration file to .toml.
     """
     import yaml
     import pytomlpp
     yaml_path = proj_info.data_dir / 'tpr_info.yaml'
+    old_toml_path = proj_info.data_dir / 'tpr_info.toml'
+
     if yaml_path.exists():
-        proj_info.config.write_text(
+        proj_info.template.write_text(
                 pytomlpp.dumps(
                     yaml.safe_load(
                         yaml_path.read_text())))
         yaml_path.unlink()
 
+    if old_toml_path.exists():
+        old_toml_path.rename(proj_info.template)
+
+
 
 @upgrade.command()
 @click.pass_obj
-def gitignore(proj_info):
+def gitignore(proj_info: ProjectInfo) -> None:
     """Update the '.gitignore' file.
     """
-    from .filesystem import JINJA_PATH
-    proj_gen = ProjectTemplate.load_from_project(proj_info)
+    proj_gen = LoadTemplate(proj_info)
     proj_gen.write_template_with_info(proj_info,
             JINJA_PATH.gitignore,
             proj_info.gitignore,
@@ -394,14 +416,12 @@ def gitignore(proj_info):
 
 @upgrade.command()
 @click.pass_obj
-def clean(proj_info):
+def clean(proj_info: ProjectInfo) -> None:
     """Cleans the project directory.
 
     Currently not fully implemented.
     """
-    # also clean up old stuff which might not be needed? e.g. any macro files etc. that are not
-    # linked, for example
+    # also clean up old stuff which might not be needed? e.g. any macro files
+    # etc. that are not linked, for example
     proj_info.validate(exists=True)
     proj_info.clear_temp()
-
-
